<?php

namespace SimpleBus\SymfonyBridge;

use SimpleBus\SymfonyBridge\DependencyInjection\Compiler\AddMiddlewareTags;
use SimpleBus\SymfonyBridge\DependencyInjection\Compiler\AutoRegister;
use SimpleBus\SymfonyBridge\DependencyInjection\Compiler\ConfigureMiddlewares;
use SimpleBus\SymfonyBridge\DependencyInjection\Compiler\RegisterMessageRecorders;
use SimpleBus\SymfonyBridge\DependencyInjection\Compiler\RegisterSubscribers;
use SimpleBus\SymfonyBridge\DependencyInjection\EventBusExtension;
use Symfony\Component\DependencyInjection\Compiler\PassConfig;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\HttpKernel\Bundle\Bundle;

class SimpleBusEventBusBundle extends Bundle
{
    use RequiresOtherBundles;

    private string $configurationAlias;

    public function __construct(string $alias = 'event_bus')
    {
        $this->configurationAlias = $alias;
    }

    public function build(ContainerBuilder $container): void
    {
        $this->checkRequirements(['SimpleBusCommandBusBundle'], $container);

        $container->addCompilerPass(
            new AutoRegister('event_subscriber', 'subscribes_to'),
            PassConfig::TYPE_BEFORE_OPTIMIZATION,
            10
        );

        $container->addCompilerPass(
            new ConfigureMiddlewares(
                'event_bus',
                'event_bus_middleware'
            )
        );

        $container->addCompilerPass(
            new RegisterMessageRecorders(
                'simple_bus.event_bus.aggregates_recorded_messages',
                'event_recorder'
            )
        );

        $container->addCompilerPass(
            new RegisterSubscribers(
                'simple_bus.event_bus.event_subscribers_collection',
                'simple_bus.event_bus.event_subscribers_service_locator',
                'event_subscriber',
                'subscribes_to'
            )
        );

        $container->addCompilerPass(
            new AddMiddlewareTags(
                'simple_bus.event_bus.handles_recorded_messages_middleware',
                ['command'],
                200
            ),
            PassConfig::TYPE_BEFORE_OPTIMIZATION,
            150
        );
    }

    public function getContainerExtension(): EventBusExtension
    {
        return new EventBusExtension($this->configurationAlias);
    }
}
